<?php
/*DESCRIPTION: inclus les variables habituelles pour la gestion d'un block et vérifie les droits d'accès
 *APRES l'inclusion du common_block les variables suivantes sont créées:
 *PHP:
 *      $dbconnection objet PDO de connexion à la DB
 *      $webplugin_path chemin d'accès au web plugin de repport (default: /usr/local/esia/public_html/plugin/report/
 *      $lang tableau associatif avec les variables de langue
 *      $action l'action passé dans l'url ou 'view'
 *      $gendate tableau associatif reprenant les paramètres global avec les clés suivantes:
 *               ['start']  timestamps de début
 *               ['end']    timestamps de fin
 *               ['interval'] interval en nombre de jour au format php. voir %a par rapport à la date de début  https://www.php.net/manual/fr/dateinterval.format.php
 *      $block les données du block en DB
 *      $parameters un tableau associatif reprenant les paramètres du block
 * SMARTY
 *      $block les informations du block
 *      $parameter les parametres du block (JSON)
 *      $action l'action passé dans l'url ou 'view'
 */
require_once '/usr/local/esia/public_html/plugin/report/report.conf.php';
require_once REPORT_COMMON_BLOCK;

$template_file = $webplugin_path . 'blocks/templates/device_notes.html';

define('NOTES_LIMITS', array(
    'MIN' => 1,
    'MAX' => 300,
    'DEFAULT' => 30,
));
define('NOTES_VALID_ORDERBY', array('date_desc', 'device_group', 'device_name'));

if($action == 'generate')
{
    // Block view generate
    try
    {
        // Get user Id for devices access rights
        $userId = NULL;
        if(@is_null($user) == FALSE)
        {
            // Current user for generate now
            $userId = $user->getId();
        }
        else
        {
            // User canvas creation for automatic generation
            $report_user = Model_Report::getOwnerCanevas($dbconnection, $block['include_canevas_id']);
            $userId = $report_user[0];
        }

        // Get targeted nodes with user access check
        $nodes = array();
        if(!empty($parameters['groupids']))
        {
            // If groupids found get childs nodes
            $nodes = Model_esia::getNodesByIds($dbconnection, $userId, NULL, $parameters['groupids'], Model_esia::NODE_ASSOC);
        }
        elseif(!empty($parameters['nodeids']))
        {
            // By default nodeids
            $nodes = Model_esia::getNodesByIds($dbconnection, $userId, $parameters['nodeids'], NULL, Model_esia::NODE_ASSOC);
        }

		$nodesNotesList = array();
		$columnshow = array();
		if(!isset($parameters['columnshow']))
		{
			$parameters['columnshow'] = array();
		}

		if(!empty($nodes))
		{
			// Get nodeIds
			$nodeIds = array_keys($nodes);

			// Get device notes
			if(empty($parameters['threadtypes']))
			{
				$parameters['threadtypes'] = NULL;
			}
			if(!is_numeric($parameters['limit']) || $parameters['limit'] < NOTES_LIMITS['MIN'] || $parameters['limit'] > NOTES_LIMITS['MAX'])
			{
				$parameters['limit'] = NOTES_LIMITS['DEFAULT'];
			}
			$devicesNotes = Model_esia::getDevicesNotesByThreadTypes($dbconnection, $nodeIds, $parameters['threadtypes'], $parameters['limit'], NULL, TRUE);

			// Check if we should sort by device
			switch($parameters['orderby'])
			{
				case 'device_group':
					// Check that the requested sort col is displayed or display default
					if(in_array('device_group', $parameters['columnshow']))
					{
						$deviceKeySort = 'group_path_list';
					}
					else
					{
						$deviceKeySort = '';
					}
					break;
				case 'device_name':
					$deviceKeySort = 'name';
					break;
				case 'date_desc':
				default:
					$deviceKeySort = '';
					break;
			}

			// Format data (create array like: array('firstKey' => array('secondKey' => array(noteData)))). First & Second keys are use to sort on two levels (first = device info, second = note create date)
			foreach($devicesNotes as $deviceId => $devNotes)
			{
				$firstLvlKey = 0;
				if(isset($nodes[$deviceId][$deviceKeySort]))
				{
					// Sort by device colmn
					if($deviceKeySort == 'group_path_list')
					{
						// This value is an array, sort names and use the first
						$groups = $nodes[$deviceId]['group_path_list'];
						sort($groups);
						$firstLvlKey = array_shift($groups);
					}
					elseif(is_string($nodes[$deviceId][$deviceKeySort]))
					{
						$firstLvlKey = $nodes[$deviceId][$deviceKeySort];
					}
				}

				foreach($devNotes as $note)
				{
					$arr = array(
						'device_name' => $nodes[$deviceId]['name'],
						'device_group' => $nodes[$deviceId]['group_path_list'],
						'device_ip' => $nodes[$deviceId]['ip'],
						'thread_type' => $note['thread_type'],
						'thread_title' => $note['thread_title'],
						'note_message' => $note['note_message'],
						'note_date' => $note['note_date'],
						'note_last_update' => $note['note_last_update'],
						'note_user_name' => $note['note_user_name'],
					);

					// Thread type from $lang
					$langKey = 'NODE_NOTE_THREAD_TYPE_' . $note['thread_type'];
					if(isset($lang[$langKey]))
					{
						$arr['thread_type'] = $lang[$langKey];
					}

					$nodesNotesList[$firstLvlKey][$note['note_date_ts']] = $arr;
				}
			}
			// Sort first level (device name or group) ASC
			ksort($nodesNotesList);
			// Sort second level (creation date) DESC
			foreach($nodesNotesList as $key => $value)
			{
				krsort($nodesNotesList[$key]);
			}

			// Format column show
			foreach($parameters['columnshow'] as $col)
			{
				$columnshow[$col] = TRUE;
			}
		}

        $smarty->assign('columnshow', $columnshow);
        $smarty->assign('nodesNotesList', $nodesNotesList);
        $smarty->display($template_file);
    }
    catch(EsiaDbException | Exception $e)
    {
        $smarty->assign('l_error', $e->getMessage());
    }
}
elseif($action == 'update')
{
    // Block configuration
    // Form sended update parameters & DB save
    if(!empty($_POST['blockid']))
    {
        try
        {
            // Get default block parameters from form
            $block = getPostDefaultParams($block, $_POST);
	    	$parameters = $block['parameters'];
            
            // Get others block parameters
            $parameters['nodeids'] = array();
            $parameters['groupids'] = array();
            if(!empty($_POST['elementselect']))
            {
                if($_POST['select_type_link'] == 'node')
                {
                    $parameters['nodeids'] = $_POST['elementselect'];
                }
                elseif($_POST['select_type_link'] == 'group')
                {
                    $parameters['groupids'] = $_POST['elementselect'];
                }
            }
			if(isset($_POST['columnshow']) && is_array($_POST['columnshow']))
			{
				$parameters['columnshow'] = array_keys($_POST['columnshow']);
			}
			else
			{
				$parameters['columnshow'] = array();
			}
            $parameters['threadtypes'] = $_POST['thread_types'];
            if(is_numeric($_POST['limit']) && $_POST['limit'] >= NOTES_LIMITS['MIN'] && $_POST['limit'] <= NOTES_LIMITS['MAX'])
            {
                $parameters['limit'] = $_POST['limit'];
            }
            else
            {
                $parameters['limit'] = NOTES_LIMITS['DEFAULT'];
            }
            if(!in_array($_POST['orderby'], NOTES_VALID_ORDERBY) || ($_POST['orderby'] == 'device_group' && !in_array('device_group', $parameters['columnshow'])))
            {
                // If invalid or not displayed
                $parameters['orderby'] = NOTES_VALID_ORDERBY[0];
            }
            else
            {
                $parameters['orderby'] = $_POST['orderby'];
            }

            Model_Report::UpdtReportIncludeBlock($dbconnection, $block['id'], $block['include_block_id'], $block['include_canevas_id'], $block['include_position'], json_encode($parameters));
            
            $smarty->assign('l_success', $lang['UPDATE_REPORT_SUCCESS']);
        }
        catch(EsiaDbException | Exception $e)
        {
            $smarty->assign('l_error', $e->getMessage());
        }
    }
    else
    {
        // Order by radio button value check for html page display the first time (no save)
        if(!in_array($parameters['orderby'], NOTES_VALID_ORDERBY) || ($parameters['orderby'] == 'device_group' && !in_array('device_group', $parameters['columnshow'])))
        {
            $parameters['orderby'] = NOTES_VALID_ORDERBY[0];
        }
    }
    // Html display
    
    // Link by node or group
    if(!empty($parameters['groupids']))
    {
        // If groupids found select by node groups
        $smarty->assign('linkType', 'group');
    }
    else
    {
        // By default node
        $smarty->assign('linkType', 'node');
    }

    // Types list
    $typesList = array();
    foreach(Model_esia::DEVICE_NOTES_THREAD_TYPES as $type)
    {
        $arr = array(
            'value' => $type,
            'lang' => $type,
            'selected' => FALSE,
        );

        $langKey = 'NODE_NOTE_THREAD_TYPE_' . $type;
        if(isset($lang[$langKey]))
        {
            $arr['lang'] = $lang[$langKey];
        }

        if(isset($parameters['threadtypes']) && in_array($type, $parameters['threadtypes']))
        {
            $arr['selected'] = TRUE;
        }
        
        $typesList[] = $arr;
    }
    $smarty->assign('typesList', $typesList);

    // Get & assign block common styles
    assignCommonStyles($smarty, $parameters);
    
    $smarty->assign('NOTES_LIMITS', NOTES_LIMITS);
    $smarty->assign('params', $parameters);
    $smarty->assign('action', 'update');
    $smarty->display($template_file);
}
elseif($action == 'node')
{
    $list = array();
    $nodes = Model_esia::getNodeList($dbconnection, $user->getId(), NULL, Model_esia::NODE_INSTANCE);
    if(isset($parameters['nodeids']) && is_array($parameters['nodeids']))
    {
        $nodeids = $parameters['nodeids'];
    }
    else
    {
        $nodeids = array();
    }

    foreach($nodes as $node)
    {
        $arr = array('id' => $node->getId());

        $groupName = $node->getGroup();
        if(empty($groupName))
        {
            $groupName = $lang['GROUPNODE_ROOT'];
        }
        $arr['parent_name'] = $groupName . '>' . $node->getName();

        if(in_array($node->getId(), $nodeids))
        {
            $arr['selected'] = 1;
        }
        $list[] = $arr;
    }

    // Sort list by name
    usort($list, function($a, $b)
    {
        return strcmp($a['parent_name'], $b['parent_name']);
    });

    echo json_encode($list);
}
elseif($action == 'group')
{
    $list = array();
    $groups = Model_esia::getGroupDevice($dbconnection, NULL, $user->getId());
    if(isset($parameters['groupids']) && is_array($parameters['groupids']))
    {
        $groupids = $parameters['groupids'];
    }
    else
    {
        $groupids = array();
    }

    foreach($groups as $group)
    {
        $arr = array('id' => $group['id'], 'parent_name' => $group['fullpath']);

        if(in_array($group['id'], $groupids))
        {
            $arr['selected'] = 1;
        }
        $list[] = $arr;
    }

    // Sort list by name
    usort($list, function($a, $b)
    {
        return strcmp($a['parent_name'], $b['parent_name']);
    });

    echo json_encode($list);
}
?>
