<?php
/*DESCRIPTION: inclus les variables habituelles pour la gestion d'un block et vérifie les droits d'accès
 *APRES l'inclusion du common_block les variables suivantes sont créées:
 *PHP:
 *      $dbconnection objet PDO de connexion à la DB
 *      $webplugin_path chemin d'accès au web plugin de repport (default: /usr/local/esia/public_html/plugin/report/
 *      $lang tableau associatif avec les variables de langue
 *      $action l'action passé dans l'url ou 'view'
 *      $gendate tableau associatif reprenant les paramètres global avec les clés suivantes:
 *               ['start']  timestamps de début
 *               ['end']    timestamps de fin
 *               ['interval'] interval en nombre de jour au format php. voir %a par rapport à la date de début  https://www.php.net/manual/fr/dateinterval.format.php
 *      $block les données du block en DB
 *      $parameters un tableau associatif reprenant les paramètres du block
 * SMARTY
 *      $block les informations du block
 *      $parameter les parametres du block (JSON)
 *      $action l'action passé dans l'url ou 'view'
 */
require_once '/usr/local/esia/public_html/plugin/report/report.conf.php';
require_once REPORT_COMMON_BLOCK;
if($action=="generate")
{
	$template_path = $path.'/plugin/report/templates';
	$param = $block['parameters'];
	$Type = $param['type'];
	$dateFormat='%H:%M';
	if(preg_match('#\-([0-9]*)d#',$gendate['interval'],$regs))
	{
		if($regs[1]>1)
		{
			$dateFormat='%d-%m';
		}
	}
	if(!empty($param['serviceid']))
	{
		$listservice = Model_esia::getServices($dbconnection,NULL,$param['serviceid']);
		$query= $dbconnection->prepare("SELECT service_id,service_name,service_parameter,service_last_stdout,
								rrd_type,rrd_regex,rrd_time,rrd_name,rrd_max,rrd_min,rrd_vertical_label,rrd_graph_color,rrd_graph_type,
								rrd_regex_threshold,rra_graph_num,rra_file_path
							FROM esia_services
							JOIN esia_service_rra_file ON service_id = rra_service_id
							JOIN esia_plugins ON plugin_id = service_plugin_id
							JOIN rrd_plugins ON rrd_plugin_id=service_plugin_id
							WHERE service_id=? AND (rra_graph_num=rrd_graph_num OR (rra_graph_num IS NULL AND rrd_graph_num IS NULL))
							ORDER BY rra_graph_num NULLS FIRST,rrd_graph_type,rrd_name;");
		$global_array = array();
		foreach($listservice as $service)
		{
			$ret = $query->execute(array($service['id']));
			if($ret===false)
			{
				$err = $query->errorInfo();
				echo "DB error: ".$err[2]."\n";
			}
			$arr_rrd=$query->fetchAll(PDO::FETCH_OBJ);
			//liste les information pour générer le graphique et les sépares suivant leurs numéros (clé rrd_graph_num)
			$list_file=array();
			foreach($arr_rrd as $row)
			{
				$select=true;
				if(array_key_exists('select_rra',$parameters))
				{
					if(is_numeric($parameters['select_rra']))
					{
						if(empty($row->rra_graph_num) && $parameters['select_rra']==0)
						{
							$select=true;
						}
						elseif($row->rra_graph_num != $parameters['select_rra'])
						{
							$select=false;
						}
					}
				}
				if($select)
				{
					$filename = RRA_DIRECTORY.$row->rra_file_path;
					if(is_file($filename) && !in_array($filename, $list_file))
					{
						$list_file[]=$filename;
					}
				}
			}
			$rrd_configuration = array();
			foreach($arr_rrd as $row)
			{
				
				$rrd_configuration[$row->rrd_name] = array('name'=>$row->rrd_name,
								'unity'=>$row->rrd_vertical_label,
								'min'=>$row->rrd_min,
								'max'=>$row->rrd_max,
								'type' => Model_esia::RRD_GRAPH_TYPES_TO_C3[$row->rrd_graph_type],
								'color'=>$row->rrd_graph_color);
			}
			$globaljsonC3 = array();
			$conso = "AVERAGE";
			if(empty($parameters['consolidation']))
			{
				$parameters['consolidation']='AVERAGE';
			}
			foreach($list_file as $key=>$file)
			{
				$ret = rrd_fetch($file, array( $parameters['consolidation'], "--start",$gendate['start']['timestamp'], "--end", $gendate['end']['timestamp']));
				//suppression des caractère NaN de rrd.
				$global = array();
				$jsontable=array();
				$dateTable = $ret['data'];
				$global['id']=$key;
				foreach ($ret['data'] as $datasource => $data)
				{
					foreach ($data as $time => $value)
					{
						$jsontable['x'][] = date('Y-m-d G:i:s',$time);
					}
					array_pop($jsontable['x']);
					break;
				}
				//$j = 0;
				foreach ($ret['data'] as $datasource => $data)
				{
					$global['parameter'][] =$rrd_configuration[$datasource];
					foreach ($data as $time => $value)
					{
						if(is_nan($value))
						{
							$value=0;
						}
						else
						{
							$value=round($value,2);
						}
						$jsontable[$datasource][] = $value;
					}
					array_pop($jsontable[$datasource]);
				}
				$global['data'] = $jsontable;
				$globaljsonC3[] = $global;
			}
			$CommonName = '';
			//echo 'ICI'.$service['name'].' '.$service['plugin_name'].' '.$CommonName;
			if(!empty($parameters['affiche_technical_name']))
			{
				$CommonName =  $service['service_name'];
			}
			else
			{
				$CommonName = Model_esia::getCommonName($lang, $service['name'], $service['check_plugin_name']);
			}
			$global_array[] = array('id'=>$service['id'] ,
									'name' => $service['device_name'].': '.$CommonName,
									'dateFormat'=>$dateFormat,
									'graphic'=>$globaljsonC3);
		}
	}
	$smarty->assign('graphic',$global_array);
	$smarty->assign('action','generate');
	$smarty->display($webplugin_path.'/blocks/templates/graphique.html');
	
}
elseif($action=="update")
{
	$template_path = $path.'/plugin/report/templates';
	$param =$block['parameters'];
	$rightWriting=$_GET['writingright'];
	if(!isset($rightWriting))
	{
		$rightWriting=$_POST['writingright'];
	}
	$smarty->assign('rightWriting',$rightWriting);
	$smarty->assign('forme',$param['forme']);
	
	if(isset($_POST['blockid'])&&!empty($_POST['blockid']))
	{
		try
		{
			$block = getPostDefaultParams($block,$_POST);
			$parameters = $block['parameters'];
			
			
			// Service
			$parameters['nodeid']=$_POST['nodes'];
			$parameters['serviceid']=$_POST['services'];
			
			// Frome
			$parameters['forme']=$_POST['forme'];
			$parameters['affiche_technical_name']=$_POST['technical_name'];
			$parameters['consolidation'] = $_POST['consolidation'];
			$parameters['select_rra']=$_POST['select_rra'];
			$paramSave = json_encode($parameters);
			
			Model_Report::UpdtReportIncludeBlock($dbconnection,$block['id'],$block['include_block_id'],$block['include_canevas_id'],$block['include_position'],$paramSave);
			
			$smarty->assign('l_success',$lang['UPDATE_DASHBOARD_SUCCESS']);
			$smarty->assign('block',$block);
		}
		catch (EsiaDbException $e)
		{
			$smarty->assign('l_error',$e->getMessage());
		}
		catch (Exception $e)
		{
			$smarty->assign('l_error',$e->getMessage());
		}
		$smarty->assign('parameters',$parameters);
		$smarty->assign('param',$parameters);
		
	}
	//recupération des styles
	assignCommonStyles($smarty,$parameters);
	$listnode = array();
	$smarty->assign('listserviceid',$parameters['serviceid']);
	$smarty->assign('adm_list_node', $list);
	$smarty->assign('action','update');
	$smarty->display($webplugin_path.'/blocks/templates/graphique.html');
}
elseif($action =="getservice")
{
	if(isset($_GET['serviceid'])&& is_numeric($_GET['serviceid']))
	{
		$services = Model_esia::getServices($dbconnection,null,$_GET['serviceid']);
		$list=array();
		$arr = array();
		foreach ($services as $service)
		{
			$common_name = Model_esia::getCommonName($lang, $service['name'],$service['check_plugin_name']);
			$rrds = Model_esia::getRRDInformationByServiceId($dbconnection,$service['id']);
			foreach($rrds as $rrd)
			{
				if(array_key_exists($rrd['num'],$arr))
				{
					$arr[$rrd['num']]['text'] = $arr[$rrd['num']]['text'].':'.$rrd['name'];
				}
				else
				{
					$arr[$rrd['num']] = array("text"=>$common_name.' > '.$rrd['name'],"value"=>$rrd['num']);
				}
				
			}
			$arr[]=array("text"=>$lang['DASHBOARD_WIDGET_GRAPH_RRA_SELECT_ALL'],"value"=>'none');
		}
		echo json_encode(array_values($arr));
		exit(0);
	}
}
?>
