function show_confirm_and_remove(canevas_id)
{
	var dialog = $( "#dialog-confirm" );	
	dialog.data("scan_id",canevas_id);
	dialog.dialog( "open" );
}

function configure_table(table_id){	
	table = $("#"+table_id);
	table.data("tablesorter").widgets.push('print');
	table.data("tablesorter").widgets.push('columnSelector');
	table.data("tablesorter").widgetOptions['columnSelector_container']=$("#"+table_id+'_columnSelector');
	
	table.tablesorterPager({
        container: $( "#pager_"+table_id ),
        output: '{startRow} to {endRow} ({filteredRows})',
        size: 100,
    });
	
	$( '#'+table_id+'_print').on('click', function(){
          table.trigger('printTable');
	});
}

function create_report_table(table_id){
	Utils.createTable("#"+table_id);
	configure_table(table_id);
}

$(document).ready(function()
{	
	if($('#canevas_list').length) {
		create_report_table("canevas_list");
	}
});

// Report utils fonctions, objects and events
var Report = {};

// Declare Report global properties
Report.data = {
	chartsCount: 0,
	chartsToImgCount: 0,
	allChartsConverted: false,
	colorPatterns: {
		states: ['#83BC00', '#FFCB1F', '#DA002E', '#575759', '#3263C5'],
	},
};

// Set report custom events
document.addEventListener('reportAllChartsConverted', e => {
	Report.data.allChartsConverted = true;
});

// Generate chart for report with image conversion and event to know when all charts have been converted into images
// Usage: like bb.generate({options});
Report.chartGenerate = function(options) {
	return bb.generate({
		...options,
		oninit: function() {
			Report.data.chartsCount++;
			if(typeof options?.oninit === 'function')
			{
				options.oninit(this);
			}
		},
		onrendered: function() {
			if(typeof options?.onrendered === 'function')
			{
				options.onrendered(this);
			}
			Utils.convertChartToImage(this).then(img => {
				Report.data.chartsToImgCount++;
				if(Report.data.chartsToImgCount >= Report.data.chartsCount)
				{
					// All charts have been converted to img
					document.dispatchEvent(new Event('reportAllChartsConverted'));
				}
			});
		},
		tooltip: {
			show: false,
		},
	});
}
// Handle overlay cursor loader on the reportOverlay element until all charts have been generated and converted into images. So the user can know when he can print the report
Report.overlayHandler = function() {
	$(document).ready(function() {
		if(Report?.data?.allChartsConverted)
		{
			// All charts are already converted
			$('.reportOverlay').remove();
		}
		else
		{
			$(document).on('reportAllChartsConverted', function() {
				// Wait for all charts to be converted into img
				$('.reportOverlay').remove();
			});
		}
	});
}

Report.generateCommonChart = function(data, bind, type, isLegend=false, size={}, options={}) {
	return Report.chartGenerate({
		bindto: '#' + bind,
		data: {
			columns: data,
			type: type,
		},
		color: {
			pattern: Report.data.colorPatterns.states,
		},
		legend: {
			show: isLegend,
		},
		size: {
			...size,
		},
		...options,
	});
}

Report.generateGaugeThreshold = function(data, bind, threshold, isLegend, size={}, options={}) {
	return Report.generateCommonChart([ data ], bind, 'gauge', isLegend, size, {
		color: {
			pattern: Report.data.colorPatterns.states,
			threshold: {
				values: threshold,
			},
		},
		gauge: {
			label: {
				format: value => value + '%',
			},
			min: 0,
			max: 100,
		},
		...options,
	});
}
Report.formsSelectPlugin = function(htmlid,htmlgroupid)
{
	if($('#'+htmlid).attr('data-init') == 1)
	{
		let groupid = $('#'+htmlgroupid).val();
		let pluginselect = $('#'+htmlid).val();
		let request = $.ajax({
			type: 'POST',
			url: '../display_report.php?type=forms&action=plugin',
			data: { groupid: JSON.stringify(groupid),ids: JSON.stringify(pluginselect) },
			dataType: 'json'
		});
		request.done(function(data) 
		{
			eui_sselect[htmlid].setData(data);
		});
	}
}
Report.formsLoadPlugin = function(htmlid,selectgroup,selected)
{
	let request = $.ajax({
		type: 'POST',
		url: '../display_report.php?type=forms&action=plugin',
		data: { groupid: JSON.stringify(selectgroup),ids: JSON.stringify(selected) },
		dataType: 'json'
	});
	request.done(function(data) 
	{
		eui_sselect[htmlid].setData(data);
		$('#'+htmlid).attr('data-init',1);
	});
}
